/*
 * Asterisk -- A telephony toolkit for Linux.
 *
 * DPMacro Application For Asterisk
 *
 * Copyright (C) 2005, Anthony Minessale II
 *
 * Anthony Minessale II <anthmct@yahoo.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 */

#include <asterisk/file.h>
#include <asterisk/logger.h>
#include <asterisk/channel.h>
#include <asterisk/pbx.h>
#include <asterisk/module.h>
#include <asterisk/app.h>
#include <asterisk/lock.h>
#include <asterisk/config.h>
#include <asterisk/astobj.h> 
#include <stdlib.h>
#include <unistd.h>
#include <string.h>


static char *tdesc = "Dial Plan Macros";
static char *conf_file = "dpmacro.conf";

STANDARD_LOCAL_USER;
LOCAL_USER_DECL;

#define CODE_LEN 512

struct dpmacro {
    ASTOBJ_COMPONENTS(struct dpmacro);
	char code[CODE_LEN];
};

static struct dpmacro_container {
    ASTOBJ_CONTAINER_COMPONENTS(struct dpmacro);
} dpmacro_list;


static struct dpmacro *init_dpmacro(struct dpmacro *dpmacro, char *name, char *code) {

	if (! dpmacro) {
		if ((dpmacro = malloc(sizeof(struct dpmacro)))) {
			memset(dpmacro, 0, sizeof(struct dpmacro));
		}
	}

	if (dpmacro) {
		ast_copy_string(dpmacro->name, name, sizeof(dpmacro->name));
		ast_copy_string(dpmacro->code, code, sizeof(dpmacro->code));
	}

	return dpmacro;
}

static int parse_config(int reloading) 
{
	struct ast_config *cfg;
	char *entry;
	struct ast_variable *v;
	struct dpmacro *dpmacro;
	int count = 0;

	if (reloading) {
		ASTOBJ_CONTAINER_MARKALL(&dpmacro_list);
	}

	if ((cfg = ast_config_load(conf_file))) {
		for (entry = ast_category_browse(cfg, NULL); entry != NULL; entry = ast_category_browse(cfg, entry)) {
			if (!strcmp(entry, "macros")) {
				for (v = ast_variable_browse(cfg, entry); v ; v = v->next) {
					count++;
					if((dpmacro = ASTOBJ_CONTAINER_FIND(&dpmacro_list, entry))) {
						init_dpmacro(dpmacro, v->name, v->value);
						ASTOBJ_UNMARK(dpmacro);
					} else if ((dpmacro = init_dpmacro(NULL, v->name, v->value))) {
						ASTOBJ_CONTAINER_LINK(&dpmacro_list, dpmacro);
					}
				}
			}

		}
		ast_config_destroy(cfg);
	}
	ASTOBJ_CONTAINER_PRUNE_MARKED(&dpmacro_list, free);
	ast_log(LOG_NOTICE, "Loaded %d Macro%s\n", count, count == 1 ? "" : "s");
	return 0;
}
#define MAX_ARGS 128

static char *function_dpmacro(struct ast_channel *chan, char *cmd, char *data, char *buf, size_t len)
{
	char *macroname, *mydata, *ret = NULL;
	struct dpmacro *dpmacro;

	if ((macroname = ast_strdupa(data))) {
		int argc = 0;
		char *argv[MAX_ARGS];

		if ((mydata = strchr(macroname, ' '))) {
			*mydata = '\0';
			mydata++;
			argc = ast_separate_app_args(mydata, ':', argv, MAX_ARGS);
		}

		if ((dpmacro = ASTOBJ_CONTAINER_FIND(&dpmacro_list, macroname))) {
			char vname[256];
			int x = 0;
			for (x = 0 ; x < argc ; x++) {
				snprintf(vname, sizeof(vname), "ARG%d", x+1);
				pbx_builtin_setvar_helper(chan, vname, argv[x]);
			}
			memset(buf, 0, len);
			pbx_substitute_variables_helper(chan, dpmacro->code, buf, len);
			ret = buf;
			/* You can comment me if you don't want the ARGx vars to go out of scope */
			for (x = 0 ; x < argc ; x++) {
				snprintf(vname, sizeof(vname), "ARG%d", x+1);
				pbx_builtin_setvar_helper(chan, vname, NULL);
			}
		}
	} else {
		ast_log(LOG_ERROR, "Memory Error!\n");
	}

	return ret;
}

struct ast_custom_function dpmacro_function = {
    .name = "DPMACRO",
    .synopsis = "Dial Plan Macro",
    .syntax = "DPMACRO(<macro> arg1:arg2..argn)",
    .read = function_dpmacro,
	.desc = "Cache expressions in dpmacro.conf and evaluate them by name from the dialplan.\n"
	"e.g. mymacro => ${IF( $[ ${ARG1} = hello] ? yay : \"no way\")}"
	"\n\n"
};

int reload(void) {
	parse_config(1);
	return 0;
}

int unload_module(void)
{
	STANDARD_HANGUP_LOCALUSERS;
	ASTOBJ_CONTAINER_DESTROYALL(&dpmacro_list, free);
	ASTOBJ_CONTAINER_DESTROY(&dpmacro_list);
	return ast_custom_function_unregister(&dpmacro_function);
}

int load_module(void)
{
	parse_config(0);
	ASTOBJ_CONTAINER_INIT(&dpmacro_list);
	return ast_custom_function_register(&dpmacro_function);
}

char *description(void)
{
	return tdesc;
}

int usecount(void)
{
	int res;
	STANDARD_USECOUNT(res);
	return res;
}

char *key()
{
	return ASTERISK_GPL_KEY;
}

